#include <cud_lib.h>
#include <vector_types.h>
//#include "cuda_runtime_api.h"
#include <channel_descriptor.h> //TO get the description of type T
#include "cutil.h"

namespace cuda_lib
{
	template<typename T, int Dimension =1>/*, enum cudaTextureReadMode = cudaReadModeElementType>*///default mode cudaReadModeElementType
	class cud_Texture
	{
		__host__ textureReference m_texReference;
		int _w,_h; //the size of the texture

		cudaArray        *m_Cuda_ArrayTex;	//Array who will have the same size as the OpenGL Texture
		//This array will be used to do ops over the texture

	public:
		
		cud_Texture(bool  norm  = 0,
			enum cudaTextureFilterMode  fMode = cudaFilterModePoint,//cudaFilterModeLinear,
			enum cudaTextureAddressMode aMode = cudaAddressModeClamp)
		{
			m_texReference.normalized     = norm;
			m_texReference.filterMode     = fMode;
			m_texReference.addressMode[0] = aMode;
			m_texReference.addressMode[1] = aMode;
			m_texReference.channelDesc    = cudaCreateChannelDesc<T>();			
		}

		cud_Texture(int norm,
			enum cudaTextureFilterMode   fMode,
			enum cudaTextureAddressMode  aMode,
			struct cudaChannelFormatDesc desc)
		{
			normalized     = norm;
			filterMode     = fMode;
			addressMode[0] = aMode;
			addressMode[1] = aMode;
			channelDesc    = desc;
		}

		~cud_Texture()
		{
			CUDA_CHECK_CALL( cudaUnbindTexture(&m_texReference) );
			CUDA_CHECK_CALL( cudaFreeArray(m_Cuda_ArrayTex)   );
		}

		//------------------
		//-- Getters
		int GetWidth()	const{return _w;}
		int GetHeight()	const{return _h;}
		cudaArray* GetArray()	const{return m_Cuda_ArrayTex;}


		void Allocate(int width,int height)
		{
			_w=width;	_h=height;
			try
			{	
				CUDA_CHECK_CALL( cudaMallocArray(&m_Cuda_ArrayTex, &m_texReference.channelDesc, _w, _h) );
			}
			catch(exception & e)
			{
				cout<<endl<<"Exception : cud_Texture::Allocate "<< e.what() <<"W*H: "<<_w<<"*"<<_h<<endl;
				int i=0; cin>>i;
			}
		}

		//-- Transfert from host to Array so to the texture
		
		//Transfert CPU memory to GPU memory (like a memset)
		friend void operator << (cud_Texture<T,Dimension>&dest,cpu_vector<T>& src)
		{
			if(dest.GetSize() != obj.GetWidth()*obj.GetHeight())
			{
				throw exception ("gpu_vector:: operator<< the dest and source dont have the same size");//obj.Allocate(dest.GetSize());
			}	
			//copy data
			CUDA_CHECK_CALL( cudaMemcpyToArray(dest.GetArray(), 0, 0, src.GetHostMemoryPointer(), src.GetSize()* sizeof(T), cudaMemcpyHostToDevice) );					
		}

		//-- Texture Binder
		void Bind()
		{
			CUDA_CHECK_CALL( cudaBindTextureToArray(m_texReference, m_Cuda_ArrayTex) );
		}


		//-- Transfert from array to CPU mem :
		// And modify the texture in consequence
		//While CUDA kernels can't  directly write to textures, this copy is inevitable
		void CudaArrayToHostMem(cpu_vector<T> & dest)
		{
			//------------
			//  A modifier, rajouter une variable de memoire GPU dans cette classe
			//------------
			gpu_vector<T> gpuTempMem;
			gpuTempMem.Allocate(_w*_h);
			dest.Allocate(_w*_h);
			//Modify the texture :=>
			CUDA_CHECK_CALL( cudaMemcpyToArray(m_Cuda_ArrayTex, 0, 0, gpuTempMem.GetDeviceMemoryPointer(), _w*_h*sizeof(T), cudaMemcpyDeviceToDevice) );
			CUDA_CHECK_CALL( cudaMemcpy(dest.GetMemoryPointer, gpuTempMem.GetDeviceMemoryPointer(), _w*_h*sizeof(T), cudaMemcpyDeviceToHost) );
		}
        

				
	};

};


//Input data texture reference
//texture<float, 2, cudaReadModeElementType> texData;
/*
//use that in a case of not ok
texture<float, 2, cudaReadModeElementType> texRef;
textureReference* texRefPtr;
cudaGetTextureReference(&texRefPtr, texRef);
cudaChannelFormatDesc channelDesc =
cudaCreateChannelDesc<float>();
cudaBindTexture(0, texRefPtr, devPtr, &channelDesc, size);*/