//-----------------------------------------------------
//	Author	:	Adoniseagle
//	Date	:	14/12/2007
//	Contact	:	adoniseagle@hotmail[dot]com
//------------------------------------------------------

#pragma once

// includes, system
#include <iostream>

// Required to include CUDA vector types
#include <vector_types.h>
#include "cutil.h"
#include "cuda_runtime_api.h"

#include <cud_lib.h>

#include "cud_cpu_vector.h"

#include <stdexcept>
using namespace std;


namespace cuda_lib
{
	template<typename T>
	class gpu_vector
	{
		typedef unsigned int uint;
		//__device__ __constant__ T test;
		T * m_d_array;	//the delocalized memory	(in GPU ram) // d like device
		uint m_size;	//the size of the array

	public:
		gpu_vector()	{m_d_array = NULL;	m_size = 0;}
		~gpu_vector()
		{
			cudaFree(m_d_array);	//Safe_call
			m_size = 0;
		}

		void Allocate(uint Size)	//Be carefull give the size without (* Size(T))
		{
			if(m_size>0) CUDA_CHECK_CALL(cudaFree(m_d_array)); //=> Free the vector
			//allocate memory in the device space
			CUDA_CHECK_CALL( cudaMalloc((void **)&m_d_array, Size* sizeof(T)) );
			m_size = Size;
		}

		T * GetDeviceMemoryPointer() {return m_d_array;}
		uint GetSize()const	{return m_size;}

		//Transfert GPU memory to Host memory (like a memcpy)
		friend void operator >> ( gpu_vector<T>&obj,cpu_vector<T>& dest)
		{
			if(dest.GetSize() != obj.GetSize())
			{
				throw exception ("gpu_vector:: operator>> the dest and source dont have the same size");
			}
			//copy data
			CUDA_CHECK_CALL( cudaMemcpy( dest.GetHostMemoryPointer(), obj.GetDeviceMemoryPointer(), obj.GetSize()* sizeof(T), cudaMemcpyDeviceToHost) );
		}

		
		//Transfert CPU memory to GPU memory (like a memset)
		friend void operator << (gpu_vector<T>&obj,cpu_vector<T>& dest)
		{
			if(dest.GetSize() != obj.GetSize())
			{
				throw exception ("gpu_vector:: operator<< the dest and source dont have the same size");
			}	
			//copy data
			CUDA_CHECK_CALL( cudaMemcpy(obj.GetDeviceMemoryPointer(),dest.GetHostMemoryPointer(),obj.GetSize()* sizeof(T),cudaMemcpyHostToDevice ));					
		}
	};
};
