//-----------------------------------------------------
//	Author	:	Adoniseagle
//	Date	:	14/12/2007
//	Contact	:	adoniseagle@hotmail[dot]com
//
// Project an easy framework to manipulate CUDA data into CPP program
//------------------------------------------------------

/* Example of integrating CUDA functions into an existing 
 * application / framework.
 * CPP code representing the existing application / framework.
 * Compiled with default CPP compiler.
 */

// includes, system
#include <iostream>
#include <iostream>
using namespace std;

//-- Libraries to have CPP Cuda integration
#include <cud_lib.h>
#include <cud_cpu_vector.h>
#include <cud_gpu_vector.h>
#include <cppintegration.h>



////////////////////////////////////////////////////////////////////////////////
// Program main
////////////////////////////////////////////////////////////////////////////////
int
main(int argc, char** argv)
{
	//--| Initialiaze Cuda librairie
	cuda_lib::InitCudaLibrary();

	//--| Create a host and device memory interface
	cuda_lib::gpu_vector<float> device_mem;
	cuda_lib::cpu_vector<float> host_mem;

	//--| Allocate memory
	host_mem.Allocate(10,true,1.0);
	device_mem.Allocate(10);

	//--| Affect data to the cpu vector
	cout<<endl;
	for(int i=0;i<10;++i)
		cout<<" "<<host_mem[i];

	try
	{
		//--| Transfert CPU data to GPU data
		device_mem<<host_mem;

		//--| Test a little computation : (vector+=4)
		cuda_lib::CallKernelFloat(device_mem.GetDeviceMemoryPointer(),device_mem.GetSize());

		//--| Transfert GPU data to CPU data
		device_mem>>host_mem;

		//--| Display the result :
		cout<<endl;
		for(int i=0;i<10;++i)
		cout<<" "<<host_mem[i];
	}
	catch(exception & e)
	{
		cout<<endl<<"Exception : "<< e.what()<<endl;
		int i=0; cin>>i;
	}
	cuda_lib::CloseCudaLibrary();


}

